/* Copyright 1996, Jan D. Wolter and Steven R. Weiss, All Rights Reserved. */

#include "backtalk.h"

#include <signal.h>
#include <pwd.h>
#if HAVE_SETRLIMIT
#if HAVE_SYS_TIME_H
#include <sys/time.h>
#else
#include <time.h>
#endif /* HAVE_SYS_TIME_H */
#include <sys/resource.h>
#endif /* HAVE_SETRLIMIT */

#include "str.h"
#include "lock.h"
#include "udb_read.h"
#include "udb_check.h"
#include "udb_edit.h"
#include "adm_check.h"
#include "adm_add.h"
#include "set_uid.h"


char *getpass();


/* LOG - Log the account creation.
 */

void log(char *lid, char *fn, char *dir)
{
char *var, *val;
FILE *fp;
time_t clock= time(0L);

	if ((fp= fopen(ACCT_LOG_FILE,"a")) == NULL)
		return;
	lock_exclusive(fileno(fp),ACCT_LOG_FILE);

	fprintf(fp,"-- %15.15s btmkuser: %s (%s)\n dir=%s\n",
		ctime(&clock)+4,lid,fn,dir);
	fclose(fp);
	unlock(ACCT_LOG_FILE);
}


void init_cmd(char *progname)
{
#if HAVE_SETRLIMIT
struct rlimit rlim;

	/* Don't dump core (dumps could contain sensitive data) */
	rlim.rlim_cur = rlim.rlim_max = 0;
	(void)setrlimit(RLIMIT_CORE, &rlim);
#endif

	if (getuid() == 0)
		set_root_uid();
	else
		set_httpd();

	if (geteuid() != SERVER_UID)
	{
		fprintf(stderr,"%s: This program should be installed setuid "
		               "to uid=%d.\n",progname,SERVER_UID);
		exit(1);
	}

	set_cfadm();
	if (geteuid() != CFADM_UID)
	{
		fprintf(stderr,"%s: You are not the conference adminstrator \n",
			progname);
		exit(1);
	}
}


main(int argc, char **argv)
{
char bf[BFSZ],err[BFSZ];
struct passwd *pw;
int i;
char *fullname= NULL, *passwd= NULL, *login= NULL;
char *passwd2;

#if NOPWEDIT
	fprintf(stderr,"%s not supported at this site.\n",argv[0]);
	exit(1);
#else
	init_cmd(argv[0]);

	for (i= 1; i < argc; i++)
	{
		if (argv[i][0] != '-' || argv[i][2] != '\0') goto usage;

		switch(argv[i][1])
		{
		case 'l':
			if (++i >= argc) goto usage;
			login= argv[i];
			if (login_bad(login,err))
			{
				printf("Login %s bad: %s",login,err);
				exit(1);
			}
			else if (current_passwd(login))
			{
				printf("Login %s already exists.\n",login);
				exit(1);
			}
			break;

		case 'n':
			if (++i >= argc) goto usage;
			fullname= argv[i];
			if (fullname_bad(fullname,err))
			{
				printf("Fullname %s bad: %s",fullname,err);
				exit(1);
			}
			break;

		case 'p':
			if (++i >= argc) goto usage;
			passwd= argv[i];
			/* Don't do sanity checking until we have login */
			break;
		default:
			goto usage;
		}
	}

	/* If no command-line login, read one in */
	while (login == NULL)
	{
		printf("Login name: ");
		if (fgets(bf,BFSZ,stdin) == NULL || bf[0] == '\n') exit(1);
		*firstin(bf,"\n")= '\0';
		if (login_bad(bf,err))
			fputs(err,stdout);
		else if (current_passwd(bf))
			printf("Login %s already exists.\n",bf);
		else
			login= strdup(bf);
	}

	/* If no command-line fullname, read one in */
	while (fullname == NULL)
	{
		printf("Full name: ");
		if (fgets(bf,BFSZ,stdin) == NULL || bf[0] == '\n') exit(1);
		*firstin(bf,"\n")= '\0';
		if (fullname_bad(bf,err))
			fputs(err,stdout);
		else
			fullname= strdup(bf);
	}

	/* Do sanity check for command-line passwords */
	if (passwd != NULL)
	{
		if (passwd_bad(passwd,login,err))
		{
			printf("Password bad: %s",err);
			exit(1);
		}
	}
	else
	{
		/* If no command-line password, read one in */
		while (passwd == NULL)
		{
			if ((passwd2= getpass("Password: ")) == NULL ||
			    passwd2[0] == '\0')
				exit(1);
			if (passwd_bad(passwd2,login,err))
			{
				fputs(err,stdout);
				continue;
			}
			passwd2= strdup(passwd2);
			if ((passwd= getpass("Password Again: ")) == NULL ||
			    passwd[0] == '\0')
				exit(1);
			if (strcmp(passwd,passwd2))
			{
				printf("Passwords don't match.\n");
				free(passwd2);
				passwd= NULL;
				continue;
			}
		}
		free(passwd2);
	}

	/* Hold some signals */
	signal(SIGINT,SIG_IGN);
	signal(SIGQUIT,SIG_IGN);

	set_cfadm();

	/* Create conference account */
	adduser(login, passwd, fullname, bf);

	/* Log the creation */
	log(login,fullname,bf);

	printf("Account %s created.\n",login);
	exit(0);
usage:
	fprintf(stderr,"usage: %s [-l <login>] [-p <password>] "
			"[-n <fullname>]\n",argv[0]);
	exit(1);
#endif
}
